package cz.upol.jj2.lab04;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * Repezentace databaze filmu s pripraenymi vyrazy pro zakladni CRUD operace
 */
public class MovieDatabase implements AutoCloseable {

	/**
	 * obtain movies' names
	 */
	private PreparedStatement getAllNames;
	/**
	 * obtain all info about movie by name and year
	 */
	private PreparedStatement getByNameStmt;
	/**
	 * insert new movie into db
	 */
	private PreparedStatement insertStmt;

	/**
	 * update movie's rating (movie given by name and year)
	 */
	private PreparedStatement updateStmt;
	/**
	 * delete movie by name and year
	 */
	private PreparedStatement deleteStmt;
	
	public MovieDatabase(Connection con) throws MovieDBException {
		try {
			getAllNames   = con.prepareStatement("SELECT name FROM movies");
			getByNameStmt = con.prepareStatement("SELECT * FROM movies WHERE (name = ?) AND (released_in = ?)");
			insertStmt    = con.prepareStatement("INSERT INTO movies (name, released_in, director, rating) VALUES (?, ?, ?, ?)");
			updateStmt    = con.prepareStatement("UPDATE movies SET rating = ? WHERE (name = ?) AND (released_in = ?)");
			deleteStmt    = con.prepareStatement("DELETE FROM movies WHERE (name = ?) AND (released_in = ?)");
		} catch (SQLException e) {
			throw new MovieDBException("Unable to initialize prepared statements.", e);
		}
	}

	/**
	 * @return names of all movies in db
	 * @throws MovieDBException for any db error
	 */
	public List<String> getMovieNames() throws MovieDBException {
		List<String> names = new ArrayList<String>();
		try (ResultSet results = getAllNames.executeQuery()) {
			while (results.next())
				names.add(results.getString(1));
		} catch (Exception e) {
			throw new MovieDBException("Unable to list all movies", e);
		}
		return names;
	}


	/**
	 * Movie by name and year
	 * @param name
	 * @param year
	 * @return Movie representation
	 * @throws MovieDBException for any db error
	 */
	public Movie getByName(String name, int year) throws MovieDBException {
		Movie movie = null;
		try {
			getByNameStmt.setString(1, name);
			getByNameStmt.setInt(2, year);
			try (ResultSet results = getByNameStmt.executeQuery()) {  //executeQuery jen pro Query výrazy
				if (results.next())
					movie = new Movie(results.getString("name"),
									  results.getInt("released_in"),
									  results.getString("director"),
									  results.getDouble("rating"));
			}
		} catch (SQLException e) {
			throw new MovieDBException("Unable to find a movie", e);
		}
		
		return movie;
	}

	/**
	 * insert new movie into db
	 * @param name name of the movie
	 * @param releasedIn relese year of the movie
	 * @param director director of the movie
	 * @param rating rating of the movie
	 * @return created movie
	 * @throws MovieDBException for any db error
	 */
	public Movie create(String name, int releasedIn, String director, double rating) throws MovieDBException {
		try {
			insertStmt.setString(1, name);
			insertStmt.setInt(2, releasedIn);
			insertStmt.setString(3, director);
			insertStmt.setDouble(4, rating);
			insertStmt.executeUpdate(); // executeUpdate pro DML výrazy
			 			
			return new Movie(name, releasedIn, director, rating);
		} catch (SQLException e) {
			throw new MovieDBException("Unable to create new movie", e);
		}
	}

	/**
	 * remove the given movie from the db
	 * @param movie
	 * @throws MovieDBException for any db error
	 */
	public void remove(Movie movie) throws MovieDBException {
		try {
			deleteStmt.setString(1, movie.getName());
			deleteStmt.setInt(2, movie.getReleasedIn());
			deleteStmt.executeUpdate();
		} catch (SQLException e) {
			throw new MovieDBException("Unable to delete movie", e);
		}
	}

	/**
	 * Commits movie changes to db
	 * @param movie
	 * @throws MovieDBException
	 */
	public void commitChanges(Movie movie) throws MovieDBException {
		try {
			updateStmt.setDouble(1, movie.getRating());
			updateStmt.setString(2, movie.getName());
			updateStmt.setInt(3, movie.getReleasedIn());
			
			updateStmt.executeUpdate();
		} catch (SQLException e) {
			throw new MovieDBException("Unable to update movie", e);
		}
	}
	
	@Override
	public void close() {
		try {
			getAllNames.close();
			getByNameStmt.close();
			insertStmt.close();
			updateStmt.close();
			deleteStmt.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}


}
